//
// RGB to YUV color space converter
//
// Copyright(c) 2002 Tatsuyuki Satoh , All rights reserved
//
// Version : beta2
//
// Important !
//
// 1. License
//
// This program is freeware for non-commercial use. 
// The user of this source must assume all legal 
// responsibility in the use of this program. 
// This is not GPL.
//
// 2. Guarantee 
//
// An author does no guarantee about this program.
// You can use this under your own risk. 
// 
// 3. A rule to distribute.
//   Do either 1) or 2).
//   1)Include a perfect original source.
//   2)Include all these headers , and add document of 
//     modification points
//
// Histry:
// 2002.11.05 : T.Satoh : beta2 , change define to parameter
//                                added customize parameter
// 2002.10.30 : T.Satoh : beta1 , Created
//

module rgb_yuv(
	R,G,B,
	Y,U,V
);

/////////////////////////////////////////////////

//
// output depth & range
//
// range scale == maximum Y value
//
parameter OUT_Y_DEPTH =   8;
parameter OUT_Y_RANGE = 113; // 714mV / 6.322mV
parameter OUT_V_DEPTH =   8;
parameter OUT_V_RANGE = OUT_Y_RANGE; // 1:1
parameter OUT_U_DEPTH =   8;
parameter OUT_U_RANGE = OUT_Y_RANGE; // 1:1

//
// internal depth (OUT_x_DEPTH <= INT_DEPTH)
//
parameter INT_DEPTH   = OUT_Y_DEPTH+4;

/////////////////////////////////////////////////


input  [7:0] R,G,B;
output [OUT_Y_DEPTH-1:0] Y;
output [OUT_U_DEPTH-1:0] U;
output [OUT_V_DEPTH-1:0] V;

parameter DP7 = INT_DEPTH-1;
parameter DP6 = INT_DEPTH-2;
parameter DP5 = INT_DEPTH-3;
parameter DP4 = INT_DEPTH-4;
parameter DP3 = INT_DEPTH-5;
parameter DP2 = INT_DEPTH-6;
parameter DP1 = INT_DEPTH-7;
parameter DP0 = INT_DEPTH-8;

wire [DP7:0] r7,g7,b7,y7,v7,u7;
wire [DP6:0] r6,g6,b6,y6,v6,u6;
wire [DP5:0] r5,g5,b5,y5,v5,u5;
wire [DP4:0] r4,g4,b4,y4,v4,u4;
wire [DP3:0] r3,g3,b3,y3,v3,u3;
wire [DP2:0] r2,g2,b2,y2,v2,u2;
wire [DP1:0] r1,g1,b1,y1,v1,u1;
wire [DP0:0] r0,g0,b0,y0,v0,u0;
//
// mask of RGB value
//
assign r0 = {(DP0+1){R[0]}};
assign g0 = {(DP0+1){G[0]}};
assign b0 = {(DP0+1){B[0]}};
assign r1 = {(DP1+1){R[1]}};
assign g1 = {(DP1+1){G[1]}};
assign b1 = {(DP1+1){B[1]}};
assign r2 = {(DP2+1){R[2]}};
assign g2 = {(DP2+1){G[2]}};
assign b2 = {(DP2+1){B[2]}};
assign r3 = {(DP3+1){R[3]}};
assign g3 = {(DP3+1){G[3]}};
assign b3 = {(DP3+1){B[3]}};
assign r4 = {(DP4+1){R[4]}};
assign g4 = {(DP4+1){G[4]}};
assign b4 = {(DP4+1){B[4]}};
assign r5 = {(DP5+1){R[5]}};
assign g5 = {(DP5+1){G[5]}};
assign b5 = {(DP5+1){B[5]}};
assign r6 = {(DP6+1){R[6]}};
assign g6 = {(DP6+1){G[6]}};
assign b6 = {(DP6+1){B[6]}};
assign r7 = {(DP7+1){R[7]}};
assign g7 = {(DP7+1){G[7]}};
assign b7 = {(DP7+1){B[7]}};

//
// Y = +0.299*R +0.587*G +0.114*B
//
parameter Y_LOW  = INT_DEPTH-OUT_Y_DEPTH;
parameter Y_RATE = OUT_Y_RANGE<<Y_LOW;
parameter V299_RATE = (299*Y_RATE/1000*128/255);
parameter V587_RATE = (587*Y_RATE/1000*128/255);
parameter V114_RATE = (144*Y_RATE/1000*128/255);

assign y0 = (r0&(V299_RATE>>7))+(g0&(V587_RATE>>7))+(b0&(V114_RATE>>7));
assign y1 = (r1&(V299_RATE>>6))+(g1&(V587_RATE>>6))+(b1&(V114_RATE>>6)) + y0;
assign y2 = (r2&(V299_RATE>>5))+(g2&(V587_RATE>>5))+(b2&(V114_RATE>>5)) + y1;
assign y3 = (r3&(V299_RATE>>4))+(g3&(V587_RATE>>4))+(b3&(V114_RATE>>4)) + y2;
assign y4 = (r4&(V299_RATE>>3))+(g4&(V587_RATE>>3))+(b4&(V114_RATE>>3)) + y3;
assign y5 = (r5&(V299_RATE>>2))+(g5&(V587_RATE>>2))+(b5&(V114_RATE>>2)) + y4;
assign y6 = (r6&(V299_RATE>>1))+(g6&(V587_RATE>>1))+(b6&(V114_RATE>>1)) + y5;
assign y7 = (r7&(V299_RATE>>0))+(g7&(V587_RATE>>0))+(b7&(V114_RATE>>0)) + y6;
assign Y = y7[DP7:Y_LOW];

//
// V = +0.615*R -0.515*G -0.100*B
//
parameter V_LOW = INT_DEPTH-OUT_V_DEPTH;
parameter V_RATE = OUT_V_RANGE<<V_LOW;
parameter P615_RATE =  (615*V_RATE/1000*128/255);
parameter M515_RATE = -(515*V_RATE/1000*128/255);
parameter M100_RATE = -(100*V_RATE/1000*128/255);

assign v0 = (r0&(P615_RATE>>7))+(g0&(M515_RATE>>7))+(b0&(M100_RATE>>7));
assign v1 = (r1&(P615_RATE>>6))+(g1&(M515_RATE>>6))+(b1&(M100_RATE>>6)) + {v0[DP0],v0};
assign v2 = (r2&(P615_RATE>>5))+(g2&(M515_RATE>>5))+(b2&(M100_RATE>>5)) + {v1[DP1],v1};
assign v3 = (r3&(P615_RATE>>4))+(g3&(M515_RATE>>4))+(b3&(M100_RATE>>4)) + {v2[DP2],v2};
assign v4 = (r4&(P615_RATE>>3))+(g4&(M515_RATE>>3))+(b4&(M100_RATE>>3)) + {v3[DP3],v3};
assign v5 = (r5&(P615_RATE>>2))+(g5&(M515_RATE>>2))+(b5&(M100_RATE>>2)) + {v4[DP4],v4};
assign v6 = (r6&(P615_RATE>>1))+(g6&(M515_RATE>>1))+(b6&(M100_RATE>>1)) + {v5[DP5],v5};
assign v7 = (r7&(P615_RATE>>0))+(g7&(M515_RATE>>0))+(b7&(M100_RATE>>0)) + {v6[DP6],v6};
assign V = v7[DP7:V_LOW];

//
// U = -0.147*R -0.289*G +0.436*B
//
parameter U_LOW = INT_DEPTH-OUT_U_DEPTH;
parameter U_RATE = OUT_U_RANGE<<U_LOW;
parameter M147_RATE = -(147*U_RATE/1000*128/255);
parameter M289_RATE = -(289*U_RATE/1000*128/255);
parameter P436_RATE =  (436*U_RATE/1000*128/255);

assign u0 = (r0&(M147_RATE>>7))+(g0&(M289_RATE>>7))+(b0&(P436_RATE>>7));
assign u1 = (r1&(M147_RATE>>6))+(g1&(M289_RATE>>6))+(b1&(P436_RATE>>6)) + {u0[DP0],u0};
assign u2 = (r2&(M147_RATE>>5))+(g2&(M289_RATE>>5))+(b2&(P436_RATE>>5)) + {u1[DP1],u1};
assign u3 = (r3&(M147_RATE>>4))+(g3&(M289_RATE>>4))+(b3&(P436_RATE>>4)) + {u2[DP2],u2};
assign u4 = (r4&(M147_RATE>>3))+(g4&(M289_RATE>>3))+(b4&(P436_RATE>>3)) + {u3[DP3],u3};
assign u5 = (r5&(M147_RATE>>2))+(g5&(M289_RATE>>2))+(b5&(P436_RATE>>2)) + {u4[DP4],u4};
assign u6 = (r6&(M147_RATE>>1))+(g6&(M289_RATE>>1))+(b6&(P436_RATE>>1)) + {u5[DP5],u5};
assign u7 = (r7&(M147_RATE>>0))+(g7&(M289_RATE>>0))+(b7&(P436_RATE>>0)) + {u6[DP6],u6};
assign U = u7[DP7:U_LOW];

endmodule

